@@ -6,14 +6,16 @@ from boto3.s3 import transfer
 from botocore.client import BaseClient
 from tqdm import tqdm
 
+from dstack.repo import RepoAddress, _repo_address_path
+
 
 def dest_file_path(key: str, output_path: Path) -> Path:
-    return output_path / "/".join(key.split("/")[4:])
+    return output_path / "/".join(key.split("/")[5:])
 
 
-def download_run_artifact_files(s3_client: BaseClient, bucket_name: str, repo_user_name: str, repo_name: str,
+def download_run_artifact_files(s3_client: BaseClient, bucket_name: str, repo_address: RepoAddress,
                                 run_name: str, output_dir: Optional[str]):
-    artifact_prefix = f"artifacts/{repo_user_name}/{repo_name}/{run_name},"
+    artifact_prefix = f"artifacts/{_repo_address_path(repo_address)}/{run_name},"
 
     output_path = Path(output_dir or os.getcwd())
 
@@ -46,15 +48,15 @@ def download_run_artifact_files(s3_client: BaseClient, bucket_name: str, repo_us
             downloader.download_file(bucket_name, key, str(file_path), callback=callback)
 
 
-def list_run_artifact_files(s3_client: BaseClient, bucket_name: str, repo_user_name: str, repo_name: str,
+def list_run_artifact_files(s3_client: BaseClient, bucket_name: str, repo_address: RepoAddress,
                             run_name: str) -> Generator[Tuple[str, str, int], None, None]:
-    artifact_prefix = f"artifacts/{repo_user_name}/{repo_name}/{run_name},"
+    artifact_prefix = f"artifacts/{_repo_address_path(repo_address)}/{run_name},"
     paginator = s3_client.get_paginator('list_objects')
     page_iterator = paginator.paginate(Bucket=bucket_name, Prefix=artifact_prefix)
     for page in page_iterator:
         for obj in (page.get("Contents") or []):
             if obj["Size"] > 0:
-                yield obj["Key"].split("/")[4], "/".join(obj["Key"].split("/")[5:]), obj["Size"]
+                yield obj["Key"].split("/")[5], "/".join(obj["Key"].split("/")[6:]), obj["Size"]
 
 
 def __remove_prefix(text, prefix):
@@ -63,7 +65,7 @@ def __remove_prefix(text, prefix):
     return text
 
 
-def upload_job_artifact_files(s3_client: BaseClient, bucket_name: str, repo_user_name: str, repo_name: str, job_id: str,
+def upload_job_artifact_files(s3_client: BaseClient, bucket_name: str, repo_address: RepoAddress, job_id: str,
                               artifact_name: str, local_path: Path):
     total_size = 0
     for root, sub_dirs, files in os.walk(local_path):
@@ -79,7 +81,7 @@ def upload_job_artifact_files(s3_client: BaseClient, bucket_name: str, repo_user
         def callback(size):
             pbar.update(size)
 
-        prefix = f"artifacts/{repo_user_name}/{repo_name}/{job_id}/{artifact_name}"
+        prefix = f"artifacts/{_repo_address_path(repo_address)}/{job_id}/{artifact_name}"
         for root, sub_dirs, files in os.walk(local_path):
             for filename in files:
                 file_path = Path(os.path.join(root, filename)).absolute()
@@ -93,9 +95,9 @@ def upload_job_artifact_files(s3_client: BaseClient, bucket_name: str, repo_user
                 )
 
 
-def list_run_artifact_files_and_folders(s3_client: BaseClient, bucket_name: str, repo_user_name: str, repo_name: str,
+def list_run_artifact_files_and_folders(s3_client: BaseClient, bucket_name: str, repo_address: RepoAddress,
                                         job_id: str, path: str) -> List[Tuple[str, bool]]:
-    prefix = f"artifacts/{repo_user_name}/{repo_name}/{job_id}/" + path + ("" if path.endswith("/") else "/")
+    prefix = f"artifacts/{_repo_address_path(repo_address)}/{job_id}/" + path + ("" if path.endswith("/") else "/")
     response = s3_client.list_objects(Bucket=bucket_name, Prefix=prefix, Delimiter="/")
     folders = []
     files = []
