@@ -5,7 +5,7 @@ from botocore.client import BaseClient
 
 from dstack.jobs import Job, JobStatus, JobHead, Requirements, GpusRequirements, JobRefId, AppSpec, DepSpec, \
     ArtifactSpec
-from dstack.repo import RepoData
+from dstack.repo import RepoData, RepoAddress, _repo_address_path
 
 
 def serialize_job(job: Job) -> dict:
@@ -13,8 +13,10 @@ def serialize_job(job: Job) -> dict:
     if job.dep_specs:
         for dep in job.dep_specs:
             deps.append({
-                "repo_user_name": dep.repo_user_name,
-                "repo_name": dep.repo_name,
+                "repo_host_name": dep.repo_address.repo_host_name,
+                "repo_port": dep.repo_address.repo_port or 0,
+                "repo_user_name": dep.repo_address.repo_user_name,
+                "repo_name": dep.repo_address.repo_name,
                 "run_name": dep.run_name,
                 "mount": dep.mount,
             })
@@ -27,6 +29,8 @@ def serialize_job(job: Job) -> dict:
             })
     job_data = {
         "job_id": job.job_id,
+        "repo_host_name": job.repo_address.repo_host_name,
+        "repo_port": job.repo_address.repo_port or 0,
         "repo_user_name": job.repo_data.repo_user_name,
         "repo_name": job.repo_data.repo_name,
         "repo_branch": job.repo_data.repo_branch,
@@ -35,6 +39,8 @@ def serialize_job(job: Job) -> dict:
         "run_name": job.run_name,
         "workflow_name": job.workflow_name or '',
         "provider_name": job.provider_name,
+        "local_repo_user_name": job.local_repo_user_name,
+        "local_repo_user_email": job.local_repo_user_email or None,
         "status": job.status.value,
         "submitted_at": job.submitted_at,
         "image_name": job.image_name,
@@ -113,11 +119,8 @@ def unserialize_job(job_data: dict) -> Job:
     dep_specs = []
     if job_data.get("deps"):
         for dep in job_data["deps"]:
-            if isinstance(dep, str):
-                dep_repo_user_name, dep_repo_name, dep_run_name = tuple(dep.split(","))
-                dep_spec = DepSpec(dep_repo_user_name, dep_repo_name, dep_run_name, False)
-            else:
-                dep_spec = DepSpec(dep["repo_user_name"], dep["repo_name"], dep["run_name"], dep.get("mount") is True)
+            dep_spec = DepSpec(RepoAddress(dep["repo_host_name"], dep.get("repo_port") or None, dep["repo_user_name"],
+                                           dep["repo_name"]), dep["run_name"], dep.get("mount") is True)
             dep_specs.append(dep_spec)
     artifact_specs = []
     if job_data.get("artifacts"):
@@ -130,10 +133,12 @@ def unserialize_job(job_data: dict) -> Job:
     master_job = JobRefId(job_data["master_job_id"]) if job_data.get("master_job_id") else None
     app_specs = ([AppSpec(a["port_index"], a["app_name"], a.get("url_path") or None, a.get("url_query_params") or None)
                   for a in (job_data.get("apps") or [])]) or None
-    job = Job(job_data["job_id"], RepoData(job_data["repo_user_name"], job_data["repo_name"],
+    job = Job(job_data["job_id"], RepoData(job_data["repo_host_name"], job_data.get("repo_port") or None,
+                                           job_data["repo_user_name"], job_data["repo_name"],
                                            job_data["repo_branch"], job_data["repo_hash"],
                                            job_data["repo_diff"] or None),
               job_data["run_name"], job_data.get("workflow_name") or None, job_data["provider_name"],
+              job_data["local_repo_user_name"], job_data.get("local_repo_user_email") or None,
               JobStatus(job_data["status"]), job_data["submitted_at"], job_data["image_name"],
               job_data.get("commands") or None, job_data["env"] or None, job_data.get("working_dir") or None,
               artifact_specs, job_data.get("port_count") or None, job_data.get("ports") or None,
@@ -143,10 +148,11 @@ def unserialize_job(job_data: dict) -> Job:
 
 
 def _job_head_key(job: Job):
-    prefix = f"jobs/{job.repo_data.repo_user_name}/{job.repo_data.repo_name}"
+    prefix = f"jobs/{_repo_address_path(job.repo_data)}"
     key = f"{prefix}/l;" \
           f"{job.job_id};" \
           f"{job.provider_name};" \
+          f"{job.local_repo_user_name};" \
           f"{job.submitted_at};" \
           f"{job.status.value};" \
           f"{','.join([a.artifact_path for a in (job.artifact_specs or [])])};" \
@@ -162,14 +168,14 @@ def create_job(s3_client: BaseClient, bucket_name: str, job: Job, counter: List[
     job.set_id(job_id)
     if create_head:
         s3_client.put_object(Body="", Bucket=bucket_name, Key=_job_head_key(job))
-    prefix = f"jobs/{job.repo_data.repo_user_name}/{job.repo_data.repo_name}"
+    prefix = f"jobs/{_repo_address_path(job.repo_data)}"
     key = f"{prefix}/{job_id}.yaml"
     s3_client.put_object(Body=yaml.dump(serialize_job(job)), Bucket=bucket_name, Key=key)
     counter[0] += 1
 
 
-def get_job(s3_client: BaseClient, bucket_name: str, repo_user_name: str, repo_name: str, job_id: str) -> Optional[Job]:
-    prefix = f"jobs/{repo_user_name}/{repo_name}"
+def get_job(s3_client: BaseClient, bucket_name: str, repo_address: RepoAddress, job_id: str) -> Optional[Job]:
+    prefix = f"jobs/{_repo_address_path(repo_address)}"
     key = f"{prefix}/{job_id}.yaml"
     try:
         obj = s3_client.get_object(Bucket=bucket_name, Key=key)
@@ -183,7 +189,7 @@ def get_job(s3_client: BaseClient, bucket_name: str, repo_user_name: str, repo_n
 
 
 def update_job(s3_client: BaseClient, bucket_name: str, job: Job):
-    prefix = f"jobs/{job.repo_data.repo_user_name}/{job.repo_data.repo_name}"
+    prefix = f"jobs/{_repo_address_path(job.repo_data)}"
     job_head_key_prefix = f"{prefix}/l;{job.job_id};"
     response = s3_client.list_objects_v2(Bucket=bucket_name, Prefix=job_head_key_prefix, MaxKeys=1)
     for obj in response["Contents"]:
@@ -194,45 +200,47 @@ def update_job(s3_client: BaseClient, bucket_name: str, job: Job):
     s3_client.put_object(Body=yaml.dump(serialize_job(job)), Bucket=bucket_name, Key=key)
 
 
-def list_job_heads(s3_client: BaseClient, bucket_name: str, repo_user_name: str, repo_name: str,
+def list_job_heads(s3_client: BaseClient, bucket_name: str, repo_address: RepoAddress,
                    run_name: Optional[str] = None) -> List[JobHead]:
-    prefix = f"jobs/{repo_user_name}/{repo_name}"
+    prefix = f"jobs/{_repo_address_path(repo_address)}"
     job_head_key_prefix = f"{prefix}/l;"
     job_head_key_run_prefix = job_head_key_prefix + run_name if run_name else job_head_key_prefix
     response = s3_client.list_objects_v2(Bucket=bucket_name, Prefix=job_head_key_run_prefix)
     job_heads = []
     if "Contents" in response:
         for obj in response["Contents"]:
-            job_id, provider_name, submitted_at, status, artifacts, app_names, tag_name = tuple(
-                obj["Key"][len(job_head_key_prefix):].split(';'))
-            run_name, workflow_name, job_index = tuple(job_id.split(','))
-            job_heads.append(JobHead(job_id, repo_user_name, repo_name, run_name, workflow_name or None, provider_name,
-                                     JobStatus(status), int(submitted_at),
-                                     artifacts.split(',') if artifacts else None, tag_name or None,
-                                     app_names.split(',') or None))
+            t = obj["Key"][len(job_head_key_prefix):].split(';')
+            if len(t) == 8:
+                job_id, provider_name, local_repo_user_name, submitted_at, status, artifacts, app_names, tag_name = tuple(
+                    t)
+                run_name, workflow_name, job_index = tuple(job_id.split(','))
+                job_heads.append(JobHead(job_id, repo_address, run_name, workflow_name or None, provider_name,
+                                         local_repo_user_name, JobStatus(status), int(submitted_at),
+                                         artifacts.split(',') if artifacts else None, tag_name or None,
+                                         app_names.split(',') or None))
     return job_heads
 
 
-def list_job_head(s3_client: BaseClient, bucket_name: str, repo_user_name: str, repo_name: str, job_id: str) \
-        -> Optional[JobHead]:
-    prefix = f"jobs/{repo_user_name}/{repo_name}"
+def list_job_head(s3_client: BaseClient, bucket_name: str, repo_address: RepoAddress, job_id: str) -> Optional[JobHead]:
+    prefix = f"jobs/{_repo_address_path(repo_address)}"
     job_head_key_prefix = f"{prefix}/l;{job_id};"
     response = s3_client.list_objects_v2(Bucket=bucket_name, Prefix=job_head_key_prefix)
     if "Contents" in response:
         for obj in response["Contents"]:
-            provider_name, submitted_at, status, artifacts, app_names, tag_name = tuple(
-                obj["Key"][len(job_head_key_prefix):].split(';'))
-            run_name, workflow_name, job_index = tuple(job_id.split(','))
-            return JobHead(job_id, repo_user_name, repo_name, run_name, workflow_name or None, provider_name,
-                           JobStatus(status), int(submitted_at),
-                           artifacts.split(',') if artifacts else None, tag_name or None,
-                           app_names.split(',') or None)
+            t = obj["Key"][len(job_head_key_prefix):].split(';')
+            if len(t) == 7:
+                provider_name, local_repo_user_name, submitted_at, status, artifacts, app_names, tag_name = tuple(t)
+                run_name, workflow_name, job_index = tuple(job_id.split(','))
+                return JobHead(job_id, repo_address, run_name, workflow_name or None, provider_name,
+                               local_repo_user_name, JobStatus(status), int(submitted_at),
+                               artifacts.split(',') if artifacts else None, tag_name or None,
+                               app_names.split(',') or None)
     return None
 
 
-def list_jobs(s3_client: BaseClient, bucket_name: str, repo_user_name: str, repo_name: str,
+def list_jobs(s3_client: BaseClient, bucket_name: str, repo_address: RepoAddress,
               run_name: Optional[str] = None) -> List[Job]:
-    job_key_run_prefix = f"jobs/{repo_user_name}/{repo_name}/{run_name},"
+    job_key_run_prefix = f"jobs/{_repo_address_path(repo_address)}/{run_name},"
     response = s3_client.list_objects_v2(Bucket=bucket_name, Prefix=job_key_run_prefix)
     jobs = []
     if "Contents" in response:
@@ -243,8 +251,8 @@ def list_jobs(s3_client: BaseClient, bucket_name: str, repo_user_name: str, repo
     return jobs
 
 
-def delete_job_head(s3_client: BaseClient, bucket_name: str, repo_user_name: str, repo_name: str, job_id: str):
-    prefix = f"jobs/{repo_user_name}/{repo_name}"
+def delete_job_head(s3_client: BaseClient, bucket_name: str, repo_address: RepoAddress, job_id: str):
+    prefix = f"jobs/{_repo_address_path(repo_address)}"
     job_head_key_prefix = f"{prefix}/l;{job_id};"
     response = s3_client.list_objects_v2(Bucket=bucket_name, Prefix=job_head_key_prefix, MaxKeys=1)
     for obj in response["Contents"]:
