@@ -24,11 +24,11 @@ def stop_func(args: Namespace):
         try:
             repo_data = load_repo_data()
             backend = load_backend()
-            job_heads = backend.list_job_heads(repo_data.repo_user_name, repo_data.repo_name, args.run_name)
+            job_heads = backend.list_job_heads(repo_data, args.run_name)
             if job_heads:
                 for job_head in job_heads:
                     if job_head.status.is_unfinished():
-                        backend.stop_job(repo_data.repo_user_name, repo_data.repo_name, job_head.job_id, args.abort)
+                        backend.stop_job(repo_data, job_head.job_id, args.abort)
             else:
                 sys.exit(f"Cannot find the run '{args.run_name}'")
             print(f"[grey58]OK[/]")
