@@ -42,7 +42,7 @@ class LabProvider(Provider):
         self._parse_base_args(args)
 
     def create_job_specs(self) -> List[JobSpec]:
-        env = dict(self.env or {})
+        env = {}
         token = uuid.uuid4().hex
         env["TOKEN"] = token
         return [JobSpec(
@@ -65,26 +65,31 @@ class LabProvider(Provider):
 
     def _image_name(self) -> str:
         cuda_is_required = self.resources and self.resources.gpus
-        cuda_image_name = f"dstackai/miniconda:{self.python}-cuda-11.1"
-        cpu_image_name = f"dstackai/miniconda:{self.python}"
+        cuda_image_name = f"dstackai/miniforge:{self.python}-cuda-11.1"
+        cpu_image_name = f"dstackai/miniforge:{self.python}"
         return cuda_image_name if cuda_is_required else cpu_image_name
 
     def _commands(self):
-        commands = [
+        commands = []
+        if self.env:
+            self._extend_commands_with_env(commands, self.env)
+        commands.extend([
+            "conda install psutil -y",
             "pip install jupyterlab" + (f"=={self.version}" if self.version else ""),
             "pip install ipywidgets",
             "jupyter nbextension enable --py widgetsnbextension",
             "mkdir -p /root/.jupyter",
             "echo \"c.ServerApp.allow_root = True\" > /root/.jupyter/jupyter_server_config.py",
+            "echo \"c.ServerApp.allow_origin = '*'\" >> /root/.jupyter/jupyter_server_config.py",
             "echo \"c.ServerApp.open_browser = False\" >> /root/.jupyter/jupyter_server_config.py",
-            "echo \"c.ServerApp.port = $JOB_PORT_0\" >> /root/.jupyter/jupyter_server_config.py",
+            "echo \"c.ServerApp.port = $PORT_0\" >> /root/.jupyter/jupyter_server_config.py",
             "echo \"c.ServerApp.token = '$TOKEN'\" >> /root/.jupyter/jupyter_server_config.py",
-            "echo \"c.ServerApp.ip = '$JOB_HOSTNAME'\" >> /root/.jupyter/jupyter_server_config.py",
-        ]
-        if self.requirements:
-            commands.append("pip install -r " + self.requirements)
+            "echo \"c.ServerApp.ip = '0.0.0.0'\" >> /root/.jupyter/jupyter_server_config.py",
+        ])
         if self.before_run:
             commands.extend(self.before_run)
+        if self.requirements:
+            commands.append("pip install -r " + self.requirements)
         commands.append(
             f"jupyter lab"
         )
