@@ -28,10 +28,8 @@ class Resources:
 
 
 class Runner:
-    def __init__(self, runner_id: str, request_id: Optional[str], resources: Resources, job: Job,
-                 secret_names: List[str]):
+    def __init__(self, runner_id: str, request_id: Optional[str], resources: Resources, job: Job):
         self.runner_id = runner_id
         self.request_id = request_id
         self.job = job
         self.resources = resources
-        self.secret_names = secret_names
